#!/usr/bin/perl
#
# file: generate.cgi 
#
# (c) Copyright J.Fossy Weinzinger 2005
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Fossy: 2005.11.03
# Fossy: 2005.11.14
# Fossy: 2005.11.19
#

use warnings;
use strict;

use Time::HiRes qw/gettimeofday tv_interval/;
use Sudoku;     # Sudoku.pm in the same directory
use SudokuCGI;  # SudokuCGI.pm in the same directory
  

#----------------------------------------------------------------------------
sub Generate()
{
  #
  # find a randum puzzle
  #
  my @square  = map { '' } 0..80;
  my @puzzle  = @{ (Sudoku::Solve(2, @square))[0] };
  my @indexes = 0..80;
  my $add = sub {
                  my $index = splice(@indexes, int(rand(@indexes)), 1);

                  $square[$index] = $puzzle[$index];
                };

  $add->() for 1..17;
                  
  $add->() while ( Sudoku::Solve(2, @square) > 1 );

  return @square;
}


#----------------------------------------------------------------------------
sub Write(@)
{
  my @puzzle = @_;
  
  print SudokuTable(sub { (p({ -class => 'cell' }, $puzzle[Sudoku::Index(@_)] || '&nbsp;'), 'normal'); });
}


#----------------------------------------------------------------------------
sub LinkP(@)
{
  my @puzzle = @_;
  my $link   = 'solve.cgi?';

  for my $row (0..8)
    {
      for my $col (0..8)
        {
          my $val = $puzzle[Sudoku::Index($row, $col)];

          $link .= ($row + 1) . ($col + 1) . '=' . $val . '&' if $val;
        }
    }

  return $link;
}

#----------------------------------------------------------------------------

print header,
      start_html( -title => 'Sudoku Generater',
                  -style => {-src => 'sudoku.css'},
                  -head  => Link({ -rel => 'shortcut icon', -href => 'sudoku.ico' })
                ),
      "\n",
      p({ -class => 'right' }, a({ -href => '.' }, '&lt;back')),
      h1('Sudoku puzzle');

my $t0     = [ gettimeofday ];
my @puzzle = Generate();
my $time   = tv_interval($t0);

Write(@puzzle);

print p('Found these ', em(Sudoku::Class), sprintf(' puzzle for you in %.3f seconds.', $time));

print start_form,
      submit('another'),
      end_form, "\n";

print p('or let iwasno.net ', a({-href => LinkP(@puzzle)}, 'solve'), ' it for you');

print hr, "\n";

GetStatistic;
UpdateStatistic('generated', Sudoku::Class);
WriteStatistic('generated');

print p({ -class => 'right' }, a({ -href => '.' }, '&lt;back')), "\n";