#!/usr/bin/perl
#
# file: sudoku.plx
#
# (c) Copyright J.Fossy Weinzinger 2005
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use warnings;
use Sudoku;

my %statistic;
my $actual;


#----------------------------------------------------------------------------
sub Read()
{
  my @square;
  my $ridx = 0;

  while ( <> )
    {
      chomp;

      my @row = split //;

      $#row = 8;
      
      @square[9 * $ridx .. 9 * $ridx + 8] = @row;
      $ridx++;

      last if $ridx >= 9;
    }
    
  print "$ARGV:\n";
  $actual = $ARGV;
  
  #
  # empty read file
  #
  while ( !eof(ARGV) )
    {
      my $line = <>;
    }
    
  return @square;
}


#----------------------------------------------------------------------------
sub Write($)
{
  my ($h,   $H,   $v,   $V,   $hv,  $HV,  $Hv,  $hV) =
     ('-',  '=', '|',   '||', '+',  '++', '+',  '++');

  for my $ridx (0..8)
    {
      # headline
      if ( $ridx % 3 ) { print +($hV . $h x 5 . ($hv . $h x 5) x 2) x 3, $hV, "\n"; }
      else             { print +($HV . $H x 5 . ($Hv . $H x 5) x 2) x 3, $HV, "\n"; }

      for my $rridx (0..2)
        {
          for my $cidx (0..8)
            {
              my $element = $_[0]->[Sudoku::Index($ridx, $cidx)];

              if ( $cidx % 3 ) { print $v, ' '; }
              else             { print $V, ' '; }

              for my $ccidx (0..2)
                {
                  my $num = $rridx*3 + $ccidx + 1;

                  print ($num == 5 ? $element : ' ');
                }

              print ' ';
            }

          print $V, "\n";
        }
    }

  print +($HV . $H x 5 . ($Hv . $H x 5) x 2) x 3, $HV, "\n\n";
}


#----------------------------------------------------------------------------
sub Puzzle()
{
  my @results = Sudoku::Solve(15, Read());

  Write($_) foreach ( @results );

  printf "found %d result%s; maxDeep = %d Measure = %s Class = %s\n\n", 
         scalar @results, 
         (@results == 1 ? '': 's'), 
         Sudoku::MaxDeep,
         Sudoku::Measure,
         Sudoku::Class;
  
  $statistic{$actual} = Sudoku::Measure . "\t-=- " . Sudoku::Class;
  
  #
  # eof() "meta"-eof for *all* files
  #
  return ! eof();
}


#----------------------------------------------------------------------------
# main

1 while Puzzle;

print "Statistik:\n";

{
  no warnings;
  
  print $_, ":\t", $statistic{$_}, "\n" for (sort { $statistic{$a} <=> $statistic{$b} } keys %statistic);
}

print "\n"; 